@@ -12,9 +12,7 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include <ignition/math/Pose3.hh>
 #include <rcutils/logging_macros.h>
-#include <sdf/sdf.hh>
 #include <urdf_world/types.h>
 #include <urdf_model/model.h>
 
@@ -23,6 +21,16 @@
 #include <string>
 #include <vector>
 
+#include <ignition/math/Pose3.hh>
+#include <sdf/Error.hh>
+#include <sdf/Collision.hh>
+#include <sdf/Geometry.hh>
+#include <sdf/Joint.hh>
+#include <sdf/JointAxis.hh>
+#include <sdf/Link.hh>
+#include <sdf/Mesh.hh>
+#include <sdf/Visual.hh>
+
 #include "sdformat_urdf/sdformat_urdf.hpp"
 
 namespace sdformat_urdf
@@ -66,20 +74,13 @@ sdformat_urdf::sdf_to_urdf(const sdf::Root & sdf_dom, sdf::Errors & errors)
       "SDFormat xml has a world; but only a single model is supported");
     return nullptr;
   }
-  if (0u == sdf_dom.ModelCount()) {
+  if (nullptr == sdf_dom.Model()) {
     errors.emplace_back(
-      sdf::ErrorCode::STRING_READ,
+      sdf::ErrorCode::ELEMENT_MISSING,
       "SDFormat xml has no models; need at least one");
     return nullptr;
   }
-  if (1u != sdf_dom.ModelCount()) {
-    errors.emplace_back(
-      sdf::ErrorCode::STRING_READ,
-      "SDFormat xml has multiple models; but only a single model is supported");
-    return nullptr;
-  }
-
-  return convert_model(*sdf_dom.ModelByIndex(0), errors);
+  return convert_model(*sdf_dom.Model(), errors);
 }
 
 urdf::ModelInterfaceSharedPtr
@@ -493,12 +494,14 @@ sdformat_urdf::convert_joint(const sdf::Joint & sdf_joint, sdf::Errors & errors)
     case sdf::JointType::PRISMATIC:
       urdf_joint->type = urdf::Joint::PRISMATIC;
       break;
-    case sdf::JointType::INVALID:     // Unsupported: fall through to default
-    case sdf::JointType::BALL:        //  |
+    case sdf::JointType::UNIVERSAL:   // Unsupported: fall through to floating
+    case sdf::JointType::BALL:        //  Will require custom TF publisher
     case sdf::JointType::GEARBOX:     //  |
     case sdf::JointType::REVOLUTE2:   //  |
-    case sdf::JointType::SCREW:       //  |
-    case sdf::JointType::UNIVERSAL:   //  V
+    case sdf::JointType::SCREW:       //  V
+      urdf_joint->type = urdf::Joint::FLOATING;
+      break;
+    case sdf::JointType::INVALID:
     default:
       errors.emplace_back(
         sdf::ErrorCode::STRING_READ,
@@ -506,9 +509,15 @@ sdformat_urdf::convert_joint(const sdf::Joint & sdf_joint, sdf::Errors & errors)
       return nullptr;
   }
 
-  if (urdf::Joint::FIXED != urdf_joint->type) {
-    // Add axis info for non-fixed joints
+  if ((urdf::Joint::FIXED != urdf_joint->type) && (urdf::Joint::FLOATING != urdf_joint->type)) {
+    // Add axis info for non-fixed and non-floating joints
     const sdf::JointAxis * sdf_axis = sdf_joint.Axis(0);
+    if (nullptr == sdf_axis) {
+      errors.emplace_back(
+        sdf::ErrorCode::STRING_READ,
+        "Axes missing for joint [" + sdf_joint.Name() + "]");
+      return nullptr;
+    }
 
     // URDF expects axis to be expressed in the joint frame
     ignition::math::Vector3d axis_xyz;
